import React from 'react';
import { AppContext } from '../../../context/app'
import { AppData } from '../../../context/data'
import { WHITE, BLACK } from '../Stone'
import Wobler from '../../Wobler'

export default function Create() {
    const ctx = Definitions()

    const onClick = () => {
        if (!ctx.hasPlayers)
            return alert("Choose both black and white players");

        if (!ctx.hasCurrentUser)
            return alert("You must be one of the players");

        const request = ctx.get_GameProposalRequest()

        ctx.setFetching(true)

        postData("/api/gameproposal", request)
            .then((responce) => {
                console.log("responce", responce) // JSON data parsed by `data.json()` call
                ctx.clear_Message2Opponent()
                ctx.setFetching(false)
            });
    }

    return (
        <button
            className={'game-action create' + (ctx.isEnabled ? ' enabled' : ' disabled')}
            onClick={onClick}
        >
            <Wobler text="Leaderboard" dance={ctx.fetching} />
        </button>
    )
}

function Definitions() {
    const [ctx, dispatchCtx] = React.useContext(AppContext)
    const [data] = React.useContext(AppData)

    const isCurrentUser = data.isCurrentUser
    const whitePlayerName = ctx.newGame.whitePlayer
    const blackPlayerName = ctx.newGame.blackPlayer

    const hasPlayers = whitePlayerName !== blackPlayerName
        && whitePlayerName !== ''
        && blackPlayerName !== ''

    const hasCurrentUser = isCurrentUser(whitePlayerName) || isCurrentUser(blackPlayerName)

    const isEnabled = hasPlayers && hasCurrentUser

    const get_GameProposalRequest = () => {
        const [opponentName, opponentColor] = get_Opponent(isCurrentUser, whitePlayerName, blackPlayerName)

        return {
            opponentName,
            opponentColor,
            board: null,
            message: ctx.newGame.message
        }
    }

    return {
        hasPlayers,
        hasCurrentUser,
        isEnabled,

        fetching: ctx.newGame.fetching,
        setFetching: (status) => { dispatchCtx({ update: "newGame", fetching: status }) },

        get_GameProposalRequest,
        clear_Message2Opponent: () => { dispatchCtx({ update: "newGame", message: '' }) },
    }
}

function get_Opponent(isCurrentUser, whitePlayerName, blackPlayerName) {
    if (isCurrentUser(whitePlayerName)) {
        return [blackPlayerName, BLACK()]
    }

    if (isCurrentUser(blackPlayerName)) {
        return [whitePlayerName, WHITE()]
    }

    return ['', '']
}

async function postData(url = "", data = {}) {
    console.log("POST", url, data)

    const response = await fetch(url, {
        method: "POST",
        headers: {
            "Content-Type": "application/json",
        },
        body: JSON.stringify(data), // body data type must match "Content-Type" header
    });

    if (response.ok)
        return response.json()// parses JSON response into native JavaScript objects

    console.log("recponce", response)
    return {}
}
